from typing import Callable, Sequence
import numpy

from pathfinder.model.dot import DotMeasurement


class DotDipoleModel():
	'''
	Model of n static dipoles with a collection of voltage measurements
	at dots at different positions.

	Parameters
	----------
	dots : Sequence[Dot]
		A collection of dots representing a series of measured voltages.
	n: int
		The number of dipoles to assume.
	'''
	def __init__(self, dots: Sequence[DotMeasurement], n: int) -> None:
		self.dots = dots
		self.m = len(dots)
		self.n = n

	def __repr__(self) -> str:
		return f'DotDipoleModel({repr(list(self.dots))}, {self.n})'

	def costs(self) -> Callable[[numpy.ndarray], numpy.ndarray]:
		def costs_to_return(pt: numpy.ndarray) -> numpy.ndarray:
			return numpy.array([dot.cost(pt) for dot in self.dots])

		return costs_to_return

	def jac(self) -> Callable[[numpy.ndarray], numpy.ndarray]:
		def jac_to_return(pts: numpy.ndarray) -> numpy.ndarray:
			return numpy.array([dot.jac(pts) for dot in self.dots])

		return jac_to_return
